<?php
/**
 * @brief		Friends Application Class
 * @author		<a href='https://ipsviet.com'>onlyME</a>
 * @copyright	(c) 2020 onlyME
 * @package		Invision Community
 * @subpackage	Friends
 * @since		31 Jan 2020
 * @version
 */

namespace IPS\friends;

/**
 * Friends Application Class
 */
class _Application extends \IPS\Application
{

	/*-------------------------------------------------------------------------*/
	// Get Tab Content
	/*-------------------------------------------------------------------------*/
	public function getTab( $owner, $url, $tab, $database, $where, $joinMemWhere, $searchMemCol, $order, $idCol, $dateCol, $grid=FALSE )
	{
		# Page
		if ( !$grid )
		{
			$perpage = \IPS\Settings::i()->friends_page_popup > 0 ? \IPS\Settings::i()->friends_page_popup : 5;
		}
		else
		{
			$n = \IPS\Settings::i()->friends_page_profile > 0 ? \IPS\Settings::i()->friends_page_profile : 5;
			$perpage = ( $tab == 'friends' && $owner->member_id == \IPS\Member::loggedIn()->member_id ) ? $n-1 : $n;
		}

		$page = isset( \IPS\Request::i()->friendspage ) ? \intval( \IPS\Request::i()->friendspage ) : 1;

		if( $page < 1 )
		{
			$page = 1;
		}

		$url = $url->setQueryString( array(
			'ftab' 	=> $tab,
			'owner'	=> $owner->member_id
		) );

		# Query
		if ( \IPS\Request::i()->fname )
		{
			$url = $url->setQueryString( array( 'fname' => \IPS\Request::i()->fname ) );
			$where[] = $this->buildWhereMem( \IPS\Request::i()->fname, $searchMemCol );
		}

		$select = \IPS\Db::i()->select( '*', $database, $where, $order, array( ( $page - 1 ) * $perpage, $perpage ) );
		$select->join( array( 'core_members', 'mem' ), $joinMemWhere );

		$totalFriends = (int) \IPS\Db::i()->select( 'COUNT(*)', $database, $where )->first();

		$friends = array();
		foreach( $select as $row )
		{
			$mem = \IPS\Member::constructFromData( $row );
			if ( $mem->member_id > 0 )
			{
				$mem->req_time = $row[ $dateCol ];
				$friends[ $row[ $idCol ] ] = $mem;
			}
		}

		# Pagination
		$pagination = \IPS\Theme::i()->getTemplate( 'global', 'core', 'global' )->pagination(
			$url,
			ceil( $totalFriends / $perpage ),
			$page,
			$perpage,
			TRUE,
			'friendspage'
		);

		# Output
		if ( \IPS\Request::i()->browsePage == 1 || \IPS\Request::i()->search == 1 )
		{
			$html = \IPS\Theme::i()->getTemplate( 'friends', 'friends', 'front' )->friends( $owner, $tab, $friends, $pagination, $grid, TRUE, $totalFriends );
			\IPS\Output::i()->json( array( 'html' => $html ) );
		}

		return \IPS\Theme::i()->getTemplate( 'friends', 'friends', 'front' )->friends( $owner, $tab, $friends, $pagination, $grid, FALSE, $totalFriends );
	}

	/*-------------------------------------------------------------------------*/
	// Build WHERE for member search
	/*-------------------------------------------------------------------------*/
	public function buildWhereMem( $fname, $searchMemCol )
	{
		if ( !$fname || mb_strlen( $fname ) < 3 )
		{
			\IPS\Output::i()->json('OK');
		}

		$fname = str_replace( array( '%', '_' ), array( '\%', '\_' ), mb_strtolower( $fname ) );

		$mwhere = array( "name LIKE CONCAT(?, '%')" );
		$binds = array( $fname );

		$memfind = \IPS\Db::i()->select( '*', 'core_members', array_merge( array( implode( ' OR ', $mwhere ) ), $binds ), 'LENGTH(name) ASC', array( 0, 10 ) );
		foreach ( $memfind as $m )
		{
			$memberIds[] = $m['member_id'];
		}

		if ( !$memberIds )
		{
			\IPS\Output::i()->json('OK');
		}

		return array( $searchMemCol . ' IN(' . implode( ',', $memberIds ) . ')' );
	}


	/*-------------------------------------------------------------------------*/
	// Get mutual friends
	/*-------------------------------------------------------------------------*/
	protected static $_mutualFriends = NULL;
	
	public function getMutualFriends( $friendID, $data=NULL )
	{
		if ( $friendID == 0 )
		{
			return;
		}
		
		if ( static::$_mutualFriends[ $friendID ] !== NULL )
		{
			return static::$_mutualFriends[ $friendID ];
		}	
			
		if ( $data )
		{
			static::$_mutualFriends[ $friendID ] = $data;
		}
		else
		{
			$myFriends = iterator_to_array( \IPS\Db::i()->select( 'fi_friend', 'friends_friends', array( 'fi_owner=?', \IPS\Member::loggedIn()->member_id ) ) );
			$fiFriends = iterator_to_array( \IPS\Db::i()->select( 'fi_friend', 'friends_friends', array( 'fi_owner=?', $friendID) ) );
			$mutualIDs = array_intersect( $myFriends, $fiFriends );

			if ( $mutualIDs && \count( $mutualIDs ) > 0 )
			{
				foreach( \IPS\Db::i()->select( '*', 'core_members', array( \IPS\Db::i()->in( 'member_id', $mutualIDs, FALSE ) ) ) as $mem )
				{
					$data[ $mem['member_id'] ] = \IPS\Member::constructFromData( $mem );
				}
				static::$_mutualFriends[ $friendID ] = $data;
			}
		}
		
		return static::$_mutualFriends[ $friendID ];
	}














}
