<?php
/**
 * @brief		Notification Options
 * @author		<a href='https://www.invisioncommunity.com'>Invision Power Services, Inc.</a>
 * @copyright	(c) Invision Power Services, Inc.
 * @license		https://www.invisioncommunity.com/legal/standards/
 * @package		Invision Community
 * @subpackage	Friends
 * @since		10 Feb 2020
 */

namespace IPS\friends\extensions\core\Notifications;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !\defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * Notification Options
 */
class _Request
{
	/**
	 * Get configuration
	 *
	 * @param	\IPS\Member	$member	The member
	 * @return	array
	 */
	public static function configurationOptions( \IPS\Member $member = NULL )
	{
		return array(
			'new_friend'	=> array( 
				'type'				=> 'standard',
				'notificationTypes'	=> array( 'new_friend' ),
				'title'				=> 'notifications__new_friend',
				'showTitle'			=> TRUE,
				'default' 			=> array( 'inline', 'push', 'email' ), 
				'disabled' 			=> array() 
			),
			'new_request'	=> array( 
				'type'				=> 'standard',
				'notificationTypes'	=> array( 'new_request' ),
				'title'				=> 'notifications__new_request',
				'showTitle'			=> TRUE,
				'default' 			=> array( 'inline', 'push', 'email' ), 
				'disabled' 			=> array() 
			),
			'refuse'		=> array( 
				'type'				=> 'standard',
				'notificationTypes'	=> array( 'refuse' ),
				'title'				=> 'notifications__refuse',
				'showTitle'			=> TRUE,
				'default' 			=> array( 'inline', 'push', 'email' ), 
				'disabled' 			=> array() 
			),
			'unfriend'		=> array(
				'type'				=> 'standard',
				'notificationTypes'	=> array( 'unfriend' ),
				'title'				=> 'notifications__unfriend',
				'showTitle'			=> TRUE,
				'default' 			=> array( 'inline', 'push', 'email' ), 
				'disabled' 			=> array() 
			)
		);
	}
	

	/**
	 * Parse notification: key
	 *
	 * @param	\IPS\Notification\Inline	$notification	The notification
	 * @return	array
	 * @code
	 return array(
		 'title'		=> "Mark has replied to A Topic",	// The notification title
		 'url'			=> \IPS\Http\Url::internal( ... ),	// The URL the notification should link to
		 'content'		=> "Lorem ipsum dolar sit",			// [Optional] Any appropriate content. Do not format this like an email where the text
		 													// 	 explains what the notification is about - just include any appropriate content.
		 													// 	 For example, if the notification is about a post, set this as the body of the post.
		 'author'		=>  \IPS\Member::load( 1 ),			// [Optional] The user whose photo should be displayed for this notification
	 );
	 * @endcode
	 */
	public function parse_new_friend( \IPS\Notification\Inline $notification )
	{
		$member = $notification->item;
		
		if ( !$member )
		{
			throw new \OutOfRangeException;
		}
		
		return array(
			'title'		=> \IPS\Member::loggedIn()->language()->addToStack( 'notification__new_friend', FALSE, array( 'sprintf' => array( $member->name ) ) ),
			'url'		=> $member->url(),
			'author'	=> $member,
		);
	}
	
	public function parse_new_request( \IPS\Notification\Inline $notification )
	{
		$member = $notification->item;
		
		if ( !$member )
		{
			throw new \OutOfRangeException;
		}
		
		return array(
			'title'		=> \IPS\Member::loggedIn()->language()->addToStack( 'notification__new_request', FALSE, array( 'sprintf' => array( $member->name ) ) ),
			'url'		=> $member->url(),
			'author'	=> $member,
		);
	}	

	public function parse_refuse( \IPS\Notification\Inline $notification )
	{
		$member = $notification->item;
		
		if ( !$member )
		{
			throw new \OutOfRangeException;
		}
		
		return array(
			'title'		=> \IPS\Member::loggedIn()->language()->addToStack( 'notification__refuse', FALSE, array( 'sprintf' => array( $member->name ) ) ),
			'url'		=> $member->url(),
			'author'	=> $member,
		);
	}
	
	public function parse_unfriend( \IPS\Notification\Inline $notification )
	{
		$member = $notification->item;
		
		if ( !$member )
		{
			throw new \OutOfRangeException;
		}
		
		return array(
			'title'		=> \IPS\Member::loggedIn()->language()->addToStack( 'notification__unfriend', FALSE, array( 'sprintf' => array( $member->name ) ) ),
			'url'		=> $member->url(),
			'author'	=> $member,
		);
	}
	
}