<?php
/**
 * @brief		Profile extension: Friends
 * @author		<a href='https://www.invisioncommunity.com'>Invision Power Services, Inc.</a>
 * @copyright	(c) Invision Power Services, Inc.
 * @license		https://www.invisioncommunity.com/legal/standards/
 * @package		Invision Community
 * @subpackage	Friends
 * @since		13 Feb 2020
 */

namespace IPS\friends\extensions\core\Profile;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !\defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * @brief	Profile extension: Friends
 */
class _Friends
{
	/**
	 * Member
	 */
	protected $member;
	
	/**
	 * Constructor
	 *
	 * @param	\IPS\Member	$member	Member whose profile we are viewing
	 * @return	void
	 */
	public function __construct( \IPS\Member $member )
	{
		$this->member = $member;
	}
	
	/**
	 * Is there content to display?
	 *
	 * @return	bool
	 */
	public function showTab()
	{
		if ( ! \IPS\Member::loggedIn()->canAccessModule( \IPS\Application\Module::get( 'friends', 'friends', 'front' ) ) )
		{
			return FALSE;
		}
		
		if ( \IPS\Settings::i()->friends_allow_restrict_view_friends != 1 || $this->member->member_id == \IPS\Member::loggedIn()->member_id )
		{
			return TRUE;
		}
		
		$friends = (int) \IPS\Db::i()->select( 'COUNT(*)', 'friends_friends', array( 'fi_owner=?', $this->member->member_id ) )->first();
		if	( $friends == 0 )
		{
			return FALSE;
		}
		
		if ( \IPS\Settings::i()->friends_bypass == 1 && \IPS\Member::loggedIn()->isAdmin() )
		{
			return TRUE;
		}
		
		$friendConf = $this->member->friendSettings();

		if ( $friendConf['viewMyFriends'] == 'friends' )
		{
			if ( !$this->member->isFriend() )
			{
				return FALSE;
			}
		}
		
		if ( $friendConf['viewMyFriends'] == 'no' )
		{
			return FALSE;
		}
		
		return TRUE;
	}
	
	/**
	 * Display
	 *
	 * @return	string
	 */
	public function render()
	{
		$owner = $this->member;
		
		if ( $owner->member_id == \IPS\Member::loggedIn()->member_id )
		{
			$tabs = array('friends', 'fReqs', 'mReqs');
		}
		else
		{
			$tabs = array( 'friends' );
		}
		
		$url = $owner->url()->setQueryString( 'tab', 'node_friends_Friends' );

		$tab = \IPS\Request::i()->ftab;
		
		if ( $tab )
		{
			if ( $tab == 'fReqs' )
			{
				$database = "friends_received_requests";
				$where[] = array( 're_to=?', $owner->member_id );
				$joinMemWhere = 'mem.member_id=friends_received_requests.re_from';
				$searchMemCol = 're_from';
				$order = 're_id DESC';
				$idCol = 're_id';
				$dateCol = 're_date';
			}
			elseif ( $tab == 'mReqs' )
			{
				$database = "friends_sent_requests";
				$where[] = array( 'sent_from=?', $owner->member_id );
				$joinMemWhere = 'mem.member_id=friends_sent_requests.sent_to';
				$searchMemCol = 'sent_to';
				$order = 'sent_id DESC';
				$idCol = 'sent_id';
				$dateCol = 'sent_date';				
			}
			else
			{
				$database = "friends_friends";
				$where[] = array( 'fi_owner=?', $owner->member_id );
				$joinMemWhere = 'mem.member_id=friends_friends.fi_friend';
				$searchMemCol = 'fi_friend';
				if ( \IPS\Settings::i()->friends_sorting == 'name' )
				{
					$order = 'mem.name ASC';
				}
				else
				{
					$order = 'mem.last_activity DESC';
				}
				$idCol = 'fi_id';
				$dateCol = 'fi_date';				
			}

			$activeTabContents = \IPS\Application::load('friends')->getTab( $owner, $url, $tab, $database, $where, $joinMemWhere, $searchMemCol, $order, $idCol, $dateCol, TRUE );
		}
		
		if ( \IPS\Request::i()->isAjax() && $tab )
		{
			if( \IPS\Request::i()->isAjax() )
			{
				\IPS\Output::i()->sendOutput( $activeTabContents, 200, 'text/html', \IPS\Output::i()->httpHeaders );
			}
			else
			{
				\IPS\Output::i()->output = $activeTabContents;
			}
		}
		
		# Output
		return \IPS\Theme::i()->getTemplate( 'friends', 'friends', 'front' )->main( $this->member, $tabs, $url, $activeTabContents, TRUE );
	}
	
	
	
	
	
	
	
	
	
	
	
	
}