//<?php

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !\defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	exit;
}

class friends_hook_Conversation extends _HOOK_CLASS_
{
	/**
	 * Check if a member can receive new messages
	 *
	 * @param	\IPS\Member	$member	The member to check
	 * @param	\IPS\Member	$sender	The member sending the new message
	 * @return	bool
	 */
	public static function memberCanReceiveNewMessage( \IPS\Member $member, \IPS\Member $sender, $type='new' )
	{
		try
		{
			try
			{
				if ( \IPS\Settings::i()->friends_allow_restrict_pm == 1 )
				{
					if ( \IPS\Settings::i()->friends_bypass == 1 && $sender->isAdmin() )
					{
						return TRUE;
					}
					
					$friendConf = $member->friendSettings();
		
					if ( $friendConf['pm'] )
					{
						if ( $friendConf['pm'] == 'friends' )
						{
							if ( $member->isFriend( $sender->member_id ) )
							{
								return TRUE;
							}
							else
							{
								return FALSE;
							}
						}
						
						if ( $friendConf['pm'] == 'no' )
						{
							return FALSE;
						}
					}
				}
				return parent::memberCanReceiveNewMessage( $member, $sender, $type );
			}
			catch ( \Error | \RuntimeException $e )
			{
				if( \defined( '\IPS\DEBUG_HOOKS' ) AND \IPS\DEBUG_HOOKS )
				{
					\IPS\Log::log( $e, 'hook_exception' );
				}
	
				if ( method_exists( get_parent_class(), __FUNCTION__ ) )
				{
					return \call_user_func_array( 'parent::' . __FUNCTION__, \func_get_args() );
				}
				else
				{
					throw $e;
				}
			}
		}
		catch ( \Error | \RuntimeException $e )
		{
			if( \defined( '\IPS\DEBUG_HOOKS' ) AND \IPS\DEBUG_HOOKS )
			{
				\IPS\Log::log( $e, 'hook_exception' );
			}

			if ( method_exists( get_parent_class(), __FUNCTION__ ) )
			{
				return \call_user_func_array( 'parent::' . __FUNCTION__, \func_get_args() );
			}
			else
			{
				throw $e;
			}
		}
	}
	
	/**
	 * Can a given member create this type of content?
	 *
	 * @param	\IPS\Member	$member		The member
	 * @param	\IPS\Node\Model			$container	Container (e.g. forum) ID, if appropriate
	 * @param	bool		$showError	If TRUE, rather than returning a boolean value, will display an error
	 * @return	bool
	 */
	public static function canCreate( \IPS\Member $member, \IPS\Node\Model $container=NULL, $showError=FALSE )
	{
		try
		{
			try
			{
				if ( \IPS\Settings::i()->friends_allow_restrict_pm == 1 )
				{
					if ( \IPS\Settings::i()->friends_bypass != 1 || ! \IPS\Member::loggedIn()->isAdmin() )
					{
						if( $showError && \IPS\Request::i()->to )
						{
							$m = \IPS\Member::load( \IPS\Request::i()->to );
		
							if ( $m->member_id > 0 )
							{
								$friendConf = $m->friendSettings();
								
								if ( $friendConf['pm'] )
								{
									if ( $friendConf['pm'] == 'no' )
									{
										\IPS\Output::i()->error( \IPS\Member::loggedIn()->language()->addToStack( 'friends_error_pm_no', FALSE, array( 'sprintf' => $m->name ) ), '2FRIENDS101/1', 403, '' );
									}				
									
									if ( $friendConf['pm'] == 'friends' && !$m->isFriend( $member->member_id ) )
									{
										\IPS\Output::i()->error( \IPS\Member::loggedIn()->language()->addToStack( 'friends_error_only_pm_from_friends', FALSE, array( 'sprintf' => array( $m->name ) ) ), '2FRIENDS101/2', 403, '' );
									}
								}
							}
						}
					}
				}
				return parent::canCreate( $member, $container, $showError );
			}
			catch ( \Error | \RuntimeException $e )
			{
				if( \defined( '\IPS\DEBUG_HOOKS' ) AND \IPS\DEBUG_HOOKS )
				{
					\IPS\Log::log( $e, 'hook_exception' );
				}
	
				if ( method_exists( get_parent_class(), __FUNCTION__ ) )
				{
					return \call_user_func_array( 'parent::' . __FUNCTION__, \func_get_args() );
				}
				else
				{
					throw $e;
				}
			}
		}
		catch ( \Error | \RuntimeException $e )
		{
			if( \defined( '\IPS\DEBUG_HOOKS' ) AND \IPS\DEBUG_HOOKS )
			{
				\IPS\Log::log( $e, 'hook_exception' );
			}

			if ( method_exists( get_parent_class(), __FUNCTION__ ) )
			{
				return \call_user_func_array( 'parent::' . __FUNCTION__, \func_get_args() );
			}
			else
			{
				throw $e;
			}
		}
	}

}
