//<?php

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !\defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	exit;
}

class friends_hook_Member extends _HOOK_CLASS_
{
	/*-------------------------------------------------------------------------*/
	// Get friend Settings
	/*-------------------------------------------------------------------------*/
	protected $_fconf = NULL;
	
	public function friendSettings()
	{
		try
		{
			try
			{
				if ( $this->_fconf !== NULL )
				{
					return $this->_fconf;
				}
				
				$conf = array();
				
				try
				{
					$conf = \IPS\Db::i()->select( '*', 'friends_settings', array( 'mem=?', $this->member_id ) )->first();
				}
				catch ( \UnderflowException $e )
				{
					\IPS\Db::i()->insert( 'friends_settings', array( 'mem' => $this->member_id ) );
				}
				
				$this->_fconf = $conf;
				
				return $this->_fconf;
			}
			catch ( \Error | \RuntimeException $e )
			{
				if( \defined( '\IPS\DEBUG_HOOKS' ) AND \IPS\DEBUG_HOOKS )
				{
					\IPS\Log::log( $e, 'hook_exception' );
				}
	
				if ( method_exists( get_parent_class(), __FUNCTION__ ) )
				{
					return \call_user_func_array( 'parent::' . __FUNCTION__, \func_get_args() );
				}
				else
				{
					throw $e;
				}
			}
		}
		catch ( \Error | \RuntimeException $e )
		{
			if( \defined( '\IPS\DEBUG_HOOKS' ) AND \IPS\DEBUG_HOOKS )
			{
				\IPS\Log::log( $e, 'hook_exception' );
			}

			if ( method_exists( get_parent_class(), __FUNCTION__ ) )
			{
				return \call_user_func_array( 'parent::' . __FUNCTION__, \func_get_args() );
			}
			else
			{
				throw $e;
			}
		}
	}
	
	/*-------------------------------------------------------------------------*/
	// Is friend?
	/*-------------------------------------------------------------------------*/
	protected $_isFriend = NULL;
	
	public function isFriend( $friendID=0 )
	{
		try
		{
			try
			{
				if ( isset( $this->_isFriend ) )
				{
					return $this->_isFriend;
				}
				
				$friendID = $friendID > 0 ? $friendID : \IPS\Member::loggedIn()->member_id;
				
				try
				{
					$friend = \IPS\Db::i()->select( '*', 'friends_friends', array( 'fi_owner=? AND fi_friend=?', $this->member_id, $friendID ) )->first();
					$this->_isFriend = TRUE;
				}
				catch ( \UnderflowException $e )
				{
					$this->_isFriend = FALSE;
				}
				
				return $this->_isFriend;
			}
			catch ( \Error | \RuntimeException $e )
			{
				if( \defined( '\IPS\DEBUG_HOOKS' ) AND \IPS\DEBUG_HOOKS )
				{
					\IPS\Log::log( $e, 'hook_exception' );
				}
	
				if ( method_exists( get_parent_class(), __FUNCTION__ ) )
				{
					return \call_user_func_array( 'parent::' . __FUNCTION__, \func_get_args() );
				}
				else
				{
					throw $e;
				}
			}
		}
		catch ( \Error | \RuntimeException $e )
		{
			if( \defined( '\IPS\DEBUG_HOOKS' ) AND \IPS\DEBUG_HOOKS )
			{
				\IPS\Log::log( $e, 'hook_exception' );
			}

			if ( method_exists( get_parent_class(), __FUNCTION__ ) )
			{
				return \call_user_func_array( 'parent::' . __FUNCTION__, \func_get_args() );
			}
			else
			{
				throw $e;
			}
		}
	}
	
	/*-------------------------------------------------------------------------*/
	// Check sent request
	// 1: sent request
	// 2: received request
	// 0: no request
	/*-------------------------------------------------------------------------*/	
	public function friendRequested( $friendID )
	{	
			try
			{
				try
				{
				try
				{
					$where2 = array( 'sent_from=? AND sent_to=?', $this->member_id, $friendID );
					\IPS\Db::i()->select( '*', 'friends_sent_requests', $where2 )->first();
					return 1;
				}
				catch ( \UnderflowException $e ){}
					
				try
				{
					$where1 = array( 're_from=? AND re_to=?', $friendID, $this->member_id );
					\IPS\Db::i()->select( '*', 'friends_received_requests', $where1 )->first();
					return 2;
				}
				catch ( \UnderflowException $e ){}
		
				return 0;		
			}
			catch ( \Error | \RuntimeException $e )
			{
				if( \defined( '\IPS\DEBUG_HOOKS' ) AND \IPS\DEBUG_HOOKS )
				{
					\IPS\Log::log( $e, 'hook_exception' );
				}
	
				if ( method_exists( get_parent_class(), __FUNCTION__ ) )
				{
					return \call_user_func_array( 'parent::' . __FUNCTION__, \func_get_args() );
				}
				else
				{
					throw $e;
				}
			}
		}
		catch ( \Error | \RuntimeException $e )
		{
			if( \defined( '\IPS\DEBUG_HOOKS' ) AND \IPS\DEBUG_HOOKS )
			{
				\IPS\Log::log( $e, 'hook_exception' );
			}

			if ( method_exists( get_parent_class(), __FUNCTION__ ) )
			{
				return \call_user_func_array( 'parent::' . __FUNCTION__, \func_get_args() );
			}
			else
			{
				throw $e;
			}
		}
	}
	
	
	/*-------------------------------------------------------------------------*/
	// Number of friend requests
	/*-------------------------------------------------------------------------*/		
	protected $_numReq = NULL;
	
	public function numFriendRequests()
	{
		try
		{
			try
			{
				if ( $this->_numReq !== NULL )
				{
					return $this->_numReq;
				}
				
				$this->_numReq = (int) \IPS\Db::i()->select( 'COUNT(*)', 'friends_received_requests', array( 're_to=?', \IPS\Member::loggedIn()->member_id ) )->first();
				
				return $this->_numReq;
			}
			catch ( \Error | \RuntimeException $e )
			{
				if( \defined( '\IPS\DEBUG_HOOKS' ) AND \IPS\DEBUG_HOOKS )
				{
					\IPS\Log::log( $e, 'hook_exception' );
				}
	
				if ( method_exists( get_parent_class(), __FUNCTION__ ) )
				{
					return \call_user_func_array( 'parent::' . __FUNCTION__, \func_get_args() );
				}
				else
				{
					throw $e;
				}
			}
		}
		catch ( \Error | \RuntimeException $e )
		{
			if( \defined( '\IPS\DEBUG_HOOKS' ) AND \IPS\DEBUG_HOOKS )
			{
				\IPS\Log::log( $e, 'hook_exception' );
			}

			if ( method_exists( get_parent_class(), __FUNCTION__ ) )
			{
				return \call_user_func_array( 'parent::' . __FUNCTION__, \func_get_args() );
			}
			else
			{
				throw $e;
			}
		}
	}
	
}
