//<?php

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !\defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	exit;
}

class friends_hook_core_Stream extends _HOOK_CLASS_
{

	/**
	 * Get results
	 *
	 * @param	\IPS\Member|null	$member	The member to get the results as
	 * @return	\IPS\Content\Search\Query
	 */
	public function query( $member = NULL )
	{
		try
		{
			try
			{
				$queryFriends = false;
				
				try
				{
					$fOpt = \IPS\Db::i()->select( '*', 'friends_streams', array( 'stream_id=?', $this->id ) )->first();
					if ( $fOpt['stream_from_friend'] == 1 )
					{
						$this->ownership = 'all';
						
						$queryFriends = true;
					}
				}
				catch ( \UnderflowException $e ) {}
				
				$query = parent::query( $member );
				
				if ( $queryFriends )
				{
					$friendIDs = iterator_to_array( \IPS\Db::i()->select( 'fi_friend', 'friends_friends', array( 'fi_owner=?', \IPS\Member::loggedIn()->member_id ) ) );
					$query->filterByAuthor( $friendIDs );			
				}
				
				return $query;
			}
			catch ( \Error | \RuntimeException $e )
			{
				if( \defined( '\IPS\DEBUG_HOOKS' ) AND \IPS\DEBUG_HOOKS )
				{
					\IPS\Log::log( $e, 'hook_exception' );
				}
	
				if ( method_exists( get_parent_class(), __FUNCTION__ ) )
				{
					return \call_user_func_array( 'parent::' . __FUNCTION__, \func_get_args() );
				}
				else
				{
					throw $e;
				}
			}
		}
		catch ( \Error | \RuntimeException $e )
		{
			if( \defined( '\IPS\DEBUG_HOOKS' ) AND \IPS\DEBUG_HOOKS )
			{
				\IPS\Log::log( $e, 'hook_exception' );
			}

			if ( method_exists( get_parent_class(), __FUNCTION__ ) )
			{
				return \call_user_func_array( 'parent::' . __FUNCTION__, \func_get_args() );
			}
			else
			{
				throw $e;
			}
		}
	}
	
	/**
	 * [Node] Add/Edit Form
	 *
	 * @param	\IPS\Helpers\Form	$form			The form
	 * @param	string				$titleType		'Text' or 'Translatable
	 * @param	bool				$titleRequired	Is the title field required?
	 * @return	void
	 */
	public function form( &$form, $titleType='Translatable', $titleRequired=TRUE )
	{
		try
		{
			try
			{
				parent::form( $form, $titleType, $titleRequired );
				
				unset($form->elements['']['stream_ownership']);
				
				$ownership = $this->ownership;
		
				try
				{
					$fOpt = \IPS\Db::i()->select( '*', 'friends_streams', array( 'stream_id=?', $this->id ) )->first();
					if ( $fOpt['stream_from_friend'] == 1 )
					{
						$ownership = "friends";
					}
				}
				catch ( \UnderflowException $e ) {}
				
				$form->add( new \IPS\Helpers\Form\Radio( 'stream_ownership', $ownership, TRUE, array(
					'options' => array(
						'all'				=> 'stream_ownership_all',
						'started'			=> 'stream_ownership_started',
						'postedin'			=> 'stream_ownership_postedin',
						'custom'			=> 'stream_ownership_custom',
						'friends'			=> 'friends_stream_from_friends',
					),
					'toggles'	=> array(
						'custom'			=> array( 'stream_custom_members' )
					)
				) ) );		
			}
			catch ( \Error | \RuntimeException $e )
			{
				if( \defined( '\IPS\DEBUG_HOOKS' ) AND \IPS\DEBUG_HOOKS )
				{
					\IPS\Log::log( $e, 'hook_exception' );
				}
	
				if ( method_exists( get_parent_class(), __FUNCTION__ ) )
				{
					return \call_user_func_array( 'parent::' . __FUNCTION__, \func_get_args() );
				}
				else
				{
					throw $e;
				}
			}
		}
		catch ( \Error | \RuntimeException $e )
		{
			if( \defined( '\IPS\DEBUG_HOOKS' ) AND \IPS\DEBUG_HOOKS )
			{
				\IPS\Log::log( $e, 'hook_exception' );
			}

			if ( method_exists( get_parent_class(), __FUNCTION__ ) )
			{
				return \call_user_func_array( 'parent::' . __FUNCTION__, \func_get_args() );
			}
			else
			{
				throw $e;
			}
		}
	}
	
	/**
	 * [Node] Save Add/Edit Form
	 *
	 * @param	array	$values	Values from the form
	 * @return	void
	 */
	public function saveForm( $values )
	{
		try
		{
			try
			{
				$streamFriends = 0;
		
				if ( $values['ownership'] == "friends" )
				{
					$values['ownership'] = "all";
					$streamFriends = 1;		
				}
				
				parent::saveForm( $values );
				
				$where = array( 'stream_id=?', $this->id );
				
				try
				{
					\IPS\Db::i()->select( '*', 'friends_streams', $where )->first();
				}
				catch ( \UnderflowException $e )
				{
					\IPS\Db::i()->insert( 'friends_streams', array( 'stream_id' => $this->id, 'stream_from_friend' => 0 ) );
				}	
		
				\IPS\Db::i()->update( 'friends_streams', array( 'stream_from_friend' => (int) $streamFriends ), $where );			
			}
			catch ( \Error | \RuntimeException $e )
			{
				if( \defined( '\IPS\DEBUG_HOOKS' ) AND \IPS\DEBUG_HOOKS )
				{
					\IPS\Log::log( $e, 'hook_exception' );
				}
	
				if ( method_exists( get_parent_class(), __FUNCTION__ ) )
				{
					return \call_user_func_array( 'parent::' . __FUNCTION__, \func_get_args() );
				}
				else
				{
					throw $e;
				}
			}
		}
		catch ( \Error | \RuntimeException $e )
		{
			if( \defined( '\IPS\DEBUG_HOOKS' ) AND \IPS\DEBUG_HOOKS )
			{
				\IPS\Log::log( $e, 'hook_exception' );
			}

			if ( method_exists( get_parent_class(), __FUNCTION__ ) )
			{
				return \call_user_func_array( 'parent::' . __FUNCTION__, \func_get_args() );
			}
			else
			{
				throw $e;
			}
		}
	}
	

	/**
	 * [ActiveRecord] Delete Record
	 *
	 * @return	void
	 */
	public function delete()
	{
		try
		{
			try
			{
				\IPS\Db::i()->delete( 'friends_streams', array( 'stream_id=?', $this->id ) );	
				parent::delete();
			}
			catch ( \Error | \RuntimeException $e )
			{
				if( \defined( '\IPS\DEBUG_HOOKS' ) AND \IPS\DEBUG_HOOKS )
				{
					\IPS\Log::log( $e, 'hook_exception' );
				}
	
				if ( method_exists( get_parent_class(), __FUNCTION__ ) )
				{
					return \call_user_func_array( 'parent::' . __FUNCTION__, \func_get_args() );
				}
				else
				{
					throw $e;
				}
			}
		}
		catch ( \Error | \RuntimeException $e )
		{
			if( \defined( '\IPS\DEBUG_HOOKS' ) AND \IPS\DEBUG_HOOKS )
			{
				\IPS\Log::log( $e, 'hook_exception' );
			}

			if ( method_exists( get_parent_class(), __FUNCTION__ ) )
			{
				return \call_user_func_array( 'parent::' . __FUNCTION__, \func_get_args() );
			}
			else
			{
				throw $e;
			}
		}
	}






}
