<?php


namespace IPS\friends\modules\admin\friends;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !\defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * settings
 */
class _settings extends \IPS\Dispatcher\Controller
{
	/**
	 * @brief	Has been CSRF-protected
	 */
	public static $csrfProtected = TRUE;
				
	/**
	 * Execute
	 *
	 * @return	void
	 */
	public function execute()
	{
		\IPS\Dispatcher::i()->checkAcpPermission( 'settings_manage' );
		parent::execute();
	}

	/**
	 * ...
	 *
	 * @return	void
	 */
	protected function manage()
	{
		# Build Form
		$form = new \IPS\Helpers\Form;	
		$form->addTab( 'friends_header_general' );
		$form->add( new \IPS\Helpers\Form\Number( 'friends_page_popup', \IPS\Settings::i()->friends_page_popup, FALSE, array( 'min' => 1, 'max' => 100 ) ) );
		$form->add( new \IPS\Helpers\Form\Number( 'friends_page_profile', \IPS\Settings::i()->friends_page_profile, FALSE, array( 'min' => 10, 'max' => 100 ) ) );
		$form->add( new \IPS\Helpers\Form\Select( 'friends_sorting', \IPS\Settings::i()->friends_sorting, FALSE, array(
			'options' => array(
				'last_activity'	=> 'friends_sort_last_activity',
				'name'			=> 'friends_sort_name',
			)
		) ) );
		$form->add( new \IPS\Helpers\Form\Number( 'friends_max', \IPS\Settings::i()->friends_max, FALSE, array( 'max' => 2000 ) ) );
		$form->add( new \IPS\Helpers\Form\YesNo( 'friends_show_total', \IPS\Settings::i()->friends_show_total, FALSE, array() ) );
		$form->add( new \IPS\Helpers\Form\Number( 'friends_notify_auto_del', \IPS\Settings::i()->friends_notify_auto_del, FALSE, array( 'min' => 0 ), NULL, NULL, \IPS\Member::loggedIn()->language()->addToStack('friends_suff_days') ) );
		
		$form->addTab( 'friends_header_restriction' );
		$form->add( new \IPS\Helpers\Form\YesNo( 'friends_can_delete_request', \IPS\Settings::i()->friends_can_delete_request, FALSE, array() ) );		
		$form->add( new \IPS\Helpers\Form\YesNo( 'friends_allow_restrict_pm', \IPS\Settings::i()->friends_allow_restrict_pm, FALSE, array() ) );
		$form->add( new \IPS\Helpers\Form\YesNo( 'friends_allow_restrict_view_friends', \IPS\Settings::i()->friends_allow_restrict_view_friends, FALSE, array() ) );
		$form->add( new \IPS\Helpers\Form\YesNo( 'friends_allow_restrict_view_profile', \IPS\Settings::i()->friends_allow_restrict_view_profile, FALSE, array() ) );
		$form->add( new \IPS\Helpers\Form\YesNo( 'friends_bypass', \IPS\Settings::i()->friends_bypass, FALSE, array() ) );
		
		// $form->addTab( 'friends_header_notify' );
		// $form->add( new \IPS\Helpers\Form\YesNo( 'friends_notify_accept', \IPS\Settings::i()->friends_notify_accept, FALSE, array() ) );
		// $form->add( new \IPS\Helpers\Form\YesNo( 'friends_notify_unfriend', \IPS\Settings::i()->friends_notify_unfriend, FALSE, array() ) );
		// $form->add( new \IPS\Helpers\Form\YesNo( 'friends_notify_request', \IPS\Settings::i()->friends_notify_request, FALSE, array() ) );
		// $form->add( new \IPS\Helpers\Form\YesNo( 'friends_notify_refuse', \IPS\Settings::i()->friends_notify_refuse, FALSE, array() ) );
		
		/* Save */
		if ( $values = $form->values( TRUE ) )
		{
			$form->saveAsSettings( $values );
		}
		
		/* Output */
		\IPS\Output::i()->breadcrumb[] = array( \IPS\Http\Url::internal( "app=friends&module=friends&controller=settings" ), 'friends_settings' );
		\IPS\Output::i()->title = \IPS\Member::loggedIn()->language()->addToStack('friends_settings');	
		\IPS\Output::i()->output .= $form;
	}
	

}