<?php


namespace IPS\friends\modules\front\friends;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !\defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * friends
 */
class _friends extends \IPS\Dispatcher\Controller
{
	/**
	 * Execute
	 *
	 * @return	void
	 */
	public function execute()
	{
		parent::execute();
	}

	/**
	 * ...
	 *
	 * @return	void
	 */
	protected function manage()
	{
		$tabs = array('friends', 'fReqs', 'mReqs');
		
		$url = \IPS\Http\Url::internal( 'app=friends&module=friends&controller=friends', 'front', 'friends_friends' );
		
		if ( \IPS\Request::i()->owner > 0 )
		{
			$owner = \IPS\Member::load( \IPS\Request::i()->owner );
		}
		else $owner = \IPS\Member::loggedIn();
		
		$tab = \IPS\Request::i()->ftab;
		
		if ( $tab )
		{
			if ( $tab == 'fReqs' )
			{
				$database = "friends_received_requests";
				$where[] = array( 're_to=?', $owner->member_id );
				$joinMemWhere = 'mem.member_id=friends_received_requests.re_from';
				$searchMemCol = 're_from';
				$order = 're_id DESC';
				$idCol = 're_id';
				$dateCol = 're_date';
			}
			elseif ( $tab == 'mReqs' )
			{
				$database = "friends_sent_requests";
				$where[] = array( 'sent_from=?', $owner->member_id );
				$joinMemWhere = 'mem.member_id=friends_sent_requests.sent_to';
				$searchMemCol = 'sent_to';
				$order = 'sent_id DESC';
				$idCol = 'sent_id';
				$dateCol = 'sent_date';				
			}
			else
			{
				$database = "friends_friends";
				$where[] = array( 'fi_owner=?', $owner->member_id );
				$joinMemWhere = 'mem.member_id=friends_friends.fi_friend';
				$searchMemCol = 'fi_friend';
				if ( \IPS\Settings::i()->friends_sorting == 'name' )
				{
					$order = 'mem.name ASC';
				}
				else
				{
					$order = 'mem.last_activity DESC';
				}
				$idCol = 'fi_id';
				$dateCol = 'fi_date';				
			}

			$activeTabContents = \IPS\Application::load('friends')->getTab( $owner, $url, $tab, $database, $where, $joinMemWhere, $searchMemCol, $order, $idCol, $dateCol, FALSE );
		}

		if ( \IPS\Request::i()->isAjax() && $tab )
		{
			if( \IPS\Request::i()->isAjax() )
			{
				\IPS\Output::i()->sendOutput( $activeTabContents, 200, 'text/html', \IPS\Output::i()->httpHeaders );
			}
			else
			{
				\IPS\Output::i()->output = $activeTabContents;
			}				
		}
		
		# Output
		$html = \IPS\Theme::i()->getTemplate( 'friends', 'friends', 'front' )->main( $owner, $tabs, $url, $activeTabContents );
		
		if ( \IPS\Request::i()->_fromMenu > 0 )
		{
			\IPS\Output::i()->json( array( 'data' => $html ) );
		}
		else
		{
			\IPS\Output::i()->title = \IPS\Member::loggedIn()->language()->addToStack('friends_my_friends');
			\IPS\Output::i()->output = $html;
		}		
	}
	
	/*-------------------------------------------------------------------------*/
	// Find and Add New Friend
	/*-------------------------------------------------------------------------*/	
	protected function find()
	{
		if ( \IPS\Member::loggedIn()->member_id == 0 )
		{
			\IPS\Output::i()->error( 'friends_not_found', 'FRIENDS100/1', 404 );
		}
		
		# Form
		$lang = \IPS\Member::loggedIn()->language();
		
		$form = new \IPS\Helpers\Form('form', 'friends_add_friend');
		$form->class = 'friendsFind ipsForm_noLabels';
		
		if ( ! \IPS\Request::i()->_fromProfile )
		{
			$form->actionButtons[] = \IPS\Theme::i()->getTemplate( 'forms', 'core', 'global' )->button( "cancel", FALSE, null, 'ipsButton ipsButton_link formCancel' );
		}
		
		$form->add( new \IPS\Helpers\Form\Text( 'friends_user', NULL, FALSE, array( 
			'autocomplete' 		=> array(
				'source' 				=> 'app=friends&module=friends&controller=friends&do=member',
				'resultItemTemplate' 	=> 'friends.memberItem',
				'minimized'				=> false,
				'unique'				=> true,
				'minItems'				=> 1,
				'maxItems'				=> 1,
				'minAjaxLength'			=> 3,		
			)
		) ) );		
		
		# Submit
		if ( $values = $form->values() )
		{
			$member = $this->sendFriendRequest( $values['friends_user'] );
			
			if ( ! \IPS\Request::i()->isAjax() )
			{
				\IPS\Output::i()->redirect( \IPS\Request::i()->referrer() ?: \IPS\Http\Url::internal( '' ), $lang->addToStack('friends_request_sent') );
			}
			else
			{
				\IPS\Output::i()->json( array( 'message' => $lang->addToStack('friends_request_sent') ) );
			}			
		}
		
		# Output form
		$html = $form->customTemplate( array( \IPS\Theme::i()->getTemplate( 'forms', 'core' ), 'popupTemplate' ) );
		
		if ( ! \IPS\Request::i()->isAjax() )
		{
			\IPS\Output::i()->title = \IPS\Member::loggedIn()->language()->addToStack('friends_find');
			\IPS\Output::i()->output = $html;
		}
		else
		{
			if ( \IPS\Request::i()->_fromProfile == 1 )
			{
				\IPS\Output::i()->sendOutput( $html, 200, 'text/html', \IPS\Output::i()->httpHeaders );
			}
			else \IPS\Output::i()->json( array( 'html' => $html ) );
		}
	}

	public function member()
	{
		\IPS\Session::i()->csrfCheck();
		
		$results = array();
		
		$input = str_replace( array( '%', '_' ), array( '\%', '\_' ), mb_strtolower( \IPS\Request::i()->input ) );
		
		$where[] = array( "name LIKE CONCAT(?, '%')", $input );
		$where[] = array( "member_id != ?", \IPS\Member::loggedIn()->member_id );
		
		$select = \IPS\Db::i()->select( '*', 'core_members', $where, 'LENGTH(name) ASC', array( 0, 20 ) );
		foreach ( $select as $row )
		{
			$member = \IPS\Member::constructFromData( $row );
			
			$results[] = array(
				'id'	=> 	$member->member_id,
				'value' => 	$member->name,
				'name'	=> 	htmlspecialchars( $member->name, ENT_DISALLOWED | ENT_QUOTES, 'UTF-8', FALSE ),
				'extra'	=> 	$member->groupName,
				'photo'	=> 	(string) $member->photo,
			);
		}
				
		\IPS\Output::i()->json( $results );
	}

	/*-------------------------------------------------------------------------*/
	// Accept request
	/*-------------------------------------------------------------------------*/	
	protected function accept()
	{
		\IPS\Session::i()->csrfCheck();
		
		$friendID = (int) \IPS\Request::i()->mid;
		$friend = \IPS\Member::load( $friendID );
		
		# Check my total friends
		$mytotal = (int) \IPS\Db::i()->select( 'COUNT(*)', 'friends_friends', array( 'fi_owner=?', \IPS\Member::loggedIn()->member_id ) )->first();
		if ( $mytotal >= \IPS\Settings::i()->friends_max )
		{
			\IPS\Output::i()->error( 'friends_error_max_accept', 'FRIENDS100/2', 403 );
		}

		# Check total friends of other
		$total = (int) \IPS\Db::i()->select( 'COUNT(*)', 'friends_friends', array( 'fi_owner=?', $friendID ) )->first();
		if ( $total >= \IPS\Settings::i()->friends_max )
		{
			\IPS\Output::i()->error( 'friends_error_max_send', 'FRIENDS100/3', 403 );
		}		
		
		# Get request
		try
		{
			$where = array( 're_from=? AND re_to=?', $friendID, \IPS\Member::loggedIn()->member_id );
			$request = \IPS\Db::i()->select( '*', 'friends_received_requests', $where )->first();
		}
		catch ( \UnderflowException $e )
		{
			\IPS\Output::i()->error( 'friends_error_no_id', 'FRIENDS100/4', 403 );
		}
		
		# Check is friend?
		if ( \IPS\Member::loggedIn()->isFriend( $friendID ) )
		{
			\IPS\Output::i()->error( 'friends_error_added', 'FRIENDS100/5', 403 );
		}		
		
		# Update db
		\IPS\Db::i()->insert( 'friends_friends', array(
			'fi_owner'		=> \IPS\Member::loggedIn()->member_id,
			'fi_friend'		=> $friendID,
			'fi_time'		=> time(),
		) );
		
		\IPS\Db::i()->insert( 'friends_friends', array(
			'fi_owner'		=> $friendID,
			'fi_friend'		=> \IPS\Member::loggedIn()->member_id,
			'fi_time'		=> time(),
		) );
		
		\IPS\Db::i()->delete( 'friends_sent_requests', array( 'sent_from=? AND sent_to=?', $friendID, \IPS\Member::loggedIn()->member_id ) );	
		\IPS\Db::i()->delete( 'friends_received_requests', array( 're_id=?', $request['re_id'] ) );	

		# Send notification
		$notification = new \IPS\Notification( \IPS\Application::load('friends'), 'new_friend', \IPS\Member::loggedIn(), array( \IPS\Member::loggedIn() ) ); 
		$notification->recipients->attach( $friend ); 
		$notification->send();

		# Done
		$msg = \IPS\Member::loggedIn()->language()->addToStack('friends_request_accepted');
		
		if ( ! \IPS\Request::i()->isAjax() )
		{
			\IPS\Output::i()->redirect( \IPS\Request::i()->referrer() ?: \IPS\Http\Url::internal( '' ), $msg );
		}
		else
		{
			\IPS\Output::i()->json( array( 
				'message' 	=> $msg, 
				'reqCount' 	=> \IPS\Member::loggedIn()->numFriendRequests(),
				'btn'		=> \IPS\Request::i()->updateBtn == 1 ? \IPS\Theme::i()->getTemplate( 'friends', 'friends', 'front' )->profileBtn( $friend ) : '',
			) );	
		}			
	}	
	
	/*-------------------------------------------------------------------------*/
	// Refuse
	/*-------------------------------------------------------------------------*/	
	protected function refuse()
	{
		\IPS\Session::i()->csrfCheck();
		
		$friendID = (int) \IPS\Request::i()->mid;
		$friend = \IPS\Member::load( $friendID );
		
		# Get request
		try
		{
			$where = array( 're_from=? AND re_to=?', $friendID, \IPS\Member::loggedIn()->member_id );
			$request = \IPS\Db::i()->select( '*', 'friends_received_requests', $where )->first();
		}
		catch ( \UnderflowException $e )
		{
			\IPS\Output::i()->error( 'friends_error_no_id', 'FRIENDS100/6', 403 );
		}
			
		# Delete records
		\IPS\Db::i()->delete( 'friends_sent_requests', array( 'sent_from=? AND sent_to=?', $friendID, \IPS\Member::loggedIn()->member_id ) );
		\IPS\Db::i()->delete( 'friends_received_requests', array( 're_id=?', $request['re_id'] ) );	

		# Send notification
		$notification = new \IPS\Notification( \IPS\Application::load('friends'), 'refuse', \IPS\Member::loggedIn(), array( \IPS\Member::loggedIn() ) ); 
		$notification->recipients->attach( $friend ); 
		$notification->send();

		# Output
		$msg = \IPS\Member::loggedIn()->language()->addToStack('friends_request_deleted');
		
		if ( ! \IPS\Request::i()->isAjax() )
		{
			\IPS\Output::i()->redirect( \IPS\Request::i()->referrer() ?: \IPS\Http\Url::internal( '' ), $msg );
		}
		else
		{
			\IPS\Output::i()->json( array( 
				'message' 	=> $msg, 
				'reqCount' 	=> \IPS\Member::loggedIn()->numFriendRequests(),
				'btn'		=> \IPS\Request::i()->updateBtn == 1 ? \IPS\Theme::i()->getTemplate( 'friends', 'friends', 'front' )->profileBtn( $friend ) : '',
			) );	
		}		
	}
	
	/*-------------------------------------------------------------------------*/
	// Cancel my request
	/*-------------------------------------------------------------------------*/	
	protected function cancel()
	{
		\IPS\Session::i()->csrfCheck();
		
		if ( \IPS\Settings::i()->friends_can_delete_request != 1 )
		{
			\IPS\Output::i()->error( 'friends_error_no_perm_del_req', 'FRIENDS100/7', 403 );
		}
		
		$friendID = (int) \IPS\Request::i()->mid;
		$friend = \IPS\Member::load( $friendID );
		
		# Delete records
		\IPS\Db::i()->delete( 'friends_sent_requests', array( 'sent_from=? AND sent_to=?', \IPS\Member::loggedIn()->member_id, $friendID ) );	
		\IPS\Db::i()->delete( 'friends_received_requests', array( 're_from=? AND re_to=?', \IPS\Member::loggedIn()->member_id, $friendID ) );

		# Output
		$msg = \IPS\Member::loggedIn()->language()->addToStack('friends_request_cancelled');
			
		if ( ! \IPS\Request::i()->isAjax() )
		{
			\IPS\Output::i()->redirect( \IPS\Request::i()->referrer() ?: \IPS\Http\Url::internal( '' ), $msg );
		}
		else
		{
			\IPS\Output::i()->json( array( 
				'message' 	=> $msg, 
				'reqCount' 	=> \IPS\Member::loggedIn()->numFriendRequests(),
				'btn'		=> \IPS\Request::i()->updateBtn == 1 ? \IPS\Theme::i()->getTemplate( 'friends', 'friends', 'front' )->profileBtn( $friend ) : '',
			) );
		}		
	}	
	
	/*-------------------------------------------------------------------------*/
	// Unfriend
	/*-------------------------------------------------------------------------*/	
	protected function unfriend()
	{
		\IPS\Session::i()->csrfCheck();
		
		$friendID = (int) \IPS\Request::i()->mid;
		$friend = \IPS\Member::load( $friendID );
		
		# Check is Friend?
		if ( ! \IPS\Member::loggedIn()->isFriend( $friendID ) )
		{
			\IPS\Output::i()->error( 'friends_error_no_id', 'FRIENDS100/8', 403 );
		}
		
		# Delete records
		\IPS\Db::i()->delete( 'friends_friends', array( 'fi_owner=? AND fi_friend=?', \IPS\Member::loggedIn()->member_id, $friendID ) );	
		\IPS\Db::i()->delete( 'friends_friends', array( 'fi_friend=? AND fi_owner=?', \IPS\Member::loggedIn()->member_id, $friendID ) );
		
		# Send notification
		$notification = new \IPS\Notification( \IPS\Application::load('friends'), 'unfriend', \IPS\Member::loggedIn(), array( \IPS\Member::loggedIn() ) ); 
		$notification->recipients->attach( $friend ); 
		$notification->send();				

		# Output
		$msg = \IPS\Member::loggedIn()->language()->addToStack('friends_friend_deleted');
		
		if ( ! \IPS\Request::i()->isAjax() )
		{
			\IPS\Output::i()->redirect( \IPS\Request::i()->referrer() ?: \IPS\Http\Url::internal( '' ), $msg );
		}
		else
		{
			\IPS\Output::i()->json( array( 
				'message' 	=> $msg, 
				'reqCount' 	=> \IPS\Member::loggedIn()->numFriendRequests(),
				'btn'		=> \IPS\Request::i()->updateBtn == 1 ? \IPS\Theme::i()->getTemplate( 'friends', 'friends', 'front' )->profileBtn( $friend ) : '',
			) );	
		}			
	}
	
	/*-------------------------------------------------------------------------*/
	// Create & save a request
	/*-------------------------------------------------------------------------*/
	protected function sendFriendRequest( $memName )
	{		
		# Load & check member
		if ( $memName )
		{
			$member = \IPS\Member::load( $memName, 'name' );
		}
		else
		{
			$member = \IPS\Member::load( \IPS\Request::i()->member_id );
		}

		if ( ! $member->member_id || $member->member_id == \IPS\Member::loggedIn()->member_id )
		{
			\IPS\Output::i()->error( 'friends_not_found', 'FRIENDS100/9', 403 );
		}
		
		# Check total friends
		$mytotal = (int) \IPS\Db::i()->select( 'COUNT(*)', 'friends_friends', array( 'fi_owner=?', \IPS\Member::loggedIn()->member_id ) )->first();
		if ( $mytotal >= \IPS\Settings::i()->friends_max )
		{
			\IPS\Output::i()->error( 'friends_error_max_accept', 'FRIENDS100/10', 403 );
		}
		
		$total = (int) \IPS\Db::i()->select( 'COUNT(*)', 'friends_friends', array( 'fi_owner=?', $member->member_id ) )->first();
		if ( $total >= \IPS\Settings::i()->friends_max )
		{
			\IPS\Output::i()->error( 'friends_error_max_send', 'FRIENDS100/11', 403 );
		}
		
		# Is friend?
		if ( \IPS\Member::loggedIn()->isFriend( $member->member_id ) )
		{
			\IPS\Output::i()->error( 'friends_error_added', 'FRIENDS100/12', 403 );
		}
		
		# Request sent?
		if ( \IPS\Member::loggedIn()->friendRequested( $member->member_id ) > 0 )
		{
			\IPS\Output::i()->error( 'friends_error_req_sent', 'FRIENDS100/13', 403 );
		}
		
		\IPS\Db::i()->insert( 'friends_sent_requests', array(
			'sent_from'		=> \IPS\Member::loggedIn()->member_id,
			'sent_to'		=> $member->member_id,
			'sent_date'		=> time(),
		) );
		
		\IPS\Db::i()->insert( 'friends_received_requests', array(
			're_from'		=> \IPS\Member::loggedIn()->member_id,
			're_to'			=> $member->member_id,
			're_date'		=> time(),
		) );

		# Send notification
		$notification = new \IPS\Notification( \IPS\Application::load('friends'), 'new_request', \IPS\Member::loggedIn(), array( \IPS\Member::loggedIn() ) ); 
		$notification->recipients->attach( $member ); 
		$notification->send();	
		
		return $member;
	}
	
	/*-------------------------------------------------------------------------*/
	// Add friend
	/*-------------------------------------------------------------------------*/	
	protected function add()
	{
		\IPS\Session::i()->csrfCheck();
		
		if ( \IPS\Member::loggedIn()->member_id == 0 )
		{		
			\IPS\Output::i()->error( 'friends_not_found', 'FRIENDS100/14', 403 );
		}
		
		$member = $this->sendFriendRequest( \IPS\Request::i()->name );

		$msg = \IPS\Member::loggedIn()->language()->addToStack('friends_request_sent');
		
		if ( ! \IPS\Request::i()->isAjax() )
		{
			\IPS\Output::i()->redirect( \IPS\Request::i()->referrer() ?: \IPS\Http\Url::internal( '' ), $msg );
		}
		else
		{
			\IPS\Output::i()->json( array( 
				'message'	=> $msg,
				'btn'		=> \IPS\Request::i()->updateBtn == 1 ? \IPS\Theme::i()->getTemplate( 'friends', 'friends', 'front' )->profileBtn( $member ) : '',
			) );
		}		
	} 
	
	/*-------------------------------------------------------------------------*/
	// Settings
	/*-------------------------------------------------------------------------*/
	protected function settings()
	{
		$lang = \IPS\Member::loggedIn()->language();
		
		$conf = \IPS\Member::loggedIn()->friendSettings();
		
		$form = new \IPS\Helpers\Form('form', 'friends_form_save');
		$form->class = 'friendsConf ipsForm_vertical';
		$form->actionButtons[] = \IPS\Theme::i()->getTemplate( 'forms', 'core', 'global' )->button( "cancel", FALSE, null, 'ipsButton ipsButton_link formCancel' );
		
		if ( \IPS\Settings::i()->friends_allow_restrict_pm == 1 )
		{
			$form->add( new \IPS\Helpers\Form\Select( 'friends_conf_pm', $conf['pm'], FALSE, array(
				'options'	=> array(
					'all'		=> 'friends_conf_who_all',
					'friends'	=> 'friends_conf_who_friends',
				)
			) ) );
		}
		if ( \IPS\Settings::i()->friends_allow_restrict_view_friends == 1 )
		{
			$form->add( new \IPS\Helpers\Form\Select( 'friends_conf_viewMyFriends', $conf['viewMyFriends'], FALSE, array(
				'options'	=> array(
					'all'		=> 'friends_conf_who_all',
					'friends'	=> 'friends_conf_who_friends',
					'no'		=> 'friends_conf_who_no',
				)
			) ) );
		}
		if ( \IPS\Settings::i()->friends_allow_restrict_view_profile == 1 )
		{
			$form->add( new \IPS\Helpers\Form\Select( 'friends_conf_viewMyProfile', $conf['viewMyProfile'], FALSE, array(
				'options'	=> array(
					'all'		=> 'friends_conf_who_all',
					'friends'	=> 'friends_conf_who_friends',
					'no'		=> 'friends_conf_who_no',
				)
			) ) );
		}
		// if ( \IPS\Application::appIsEnabled( 'chatbox' ) && \IPS\Settings::i()->chatbox_bar_con == 1 && \IPS\chatbox\Member\Conversation::_canStart() )
		// {
			// $form->add( new \IPS\Helpers\Form\Select( 'friends_conf_chat', $conf['chat'], FALSE, array(
				// 'options'	=> array(
					// 'all'		=> 'friends_conf_who_all',
					// 'friends'	=> 'friends_conf_who_friends',
					// 'no'			=> 'friends_conf_who_no',
				// )
			// ) ) );			
		// }
		
		# Submit
		if ( $values = $form->values() )
		{			
			\IPS\Db::i()->update( 'friends_settings', array( 
				'pm' 			=> $values['friends_conf_pm'],
				'viewMyFriends' => $values['friends_conf_viewMyFriends'],
				'viewMyProfile' => $values['friends_conf_viewMyProfile'],
				'chat' 			=> $values['friends_conf_chat'],
			), array( 'id=?', $conf['id'] ) );
			
			\IPS\Output::i()->json( array( 'message' => $lang->addToStack('friends_conf_saved') ) );
		}
		
		# Output
		$html = $form->customTemplate( array( \IPS\Theme::i()->getTemplate( 'forms', 'core' ), 'popupTemplate' ) );
		if( \IPS\Request::i()->isAjax() )
		{
			\IPS\Output::i()->sendOutput( $html, 200, 'text/html', \IPS\Output::i()->httpHeaders );
		}
		else
		{
			\IPS\Output::i()->output = $html;
		}					
	}
	
	/*-------------------------------------------------------------------------*/
	// Display all mutual friends
	/*-------------------------------------------------------------------------*/
	protected function mutualFriends()
	{
		\IPS\Session::i()->csrfCheck();
		
		$mid = \IPS\Request::i()->mid > 0 ? \IPS\Request::i()->mid : 0;
		
		if ( $mid == 0 )
		{
			return;
		}
		
		$myFriends = iterator_to_array( \IPS\Db::i()->select( 'fi_friend', 'friends_friends', array( 'fi_owner=?', \IPS\Member::loggedIn()->member_id ) ) );
		$fiFriends = iterator_to_array( \IPS\Db::i()->select( 'fi_friend', 'friends_friends', array( 'fi_owner=?', $mid ) ) );
		$mutualIDs = array_intersect( $myFriends, $fiFriends );
		
		$where = array( \IPS\Db::i()->in( 'member_id', $mutualIDs, FALSE ) );
		
		$url = \IPS\Http\Url::internal( 'app=friends&module=friends&controller=friends&do=mutualFriends&mid=' . $mid, 'front', 'friends_friends' );

		$table = new \IPS\Helpers\Table\Db( 'core_members', $url, $where );
		$table->rowsTemplate = array( \IPS\Theme::i()->getTemplate( 'friends' ), 'mutualRows' );
		$table->sortBy = 'name';
		$table->limit = 10;

		\IPS\Output::i()->output = (string) $table;
	}		

}