<?php
/**
 * @brief		friendsAutoDelRequest Task
 * @author		<a href='https://www.invisioncommunity.com'>Invision Power Services, Inc.</a>
 * @copyright	(c) Invision Power Services, Inc.
 * @license		https://www.invisioncommunity.com/legal/standards/
 * @package		Invision Community
 * @subpackage	friends
 * @since		27 Feb 2021
 */

namespace IPS\friends\tasks;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !\defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * friendsAutoDelRequest Task
 */
class _friendsAutoDelRequest extends \IPS\Task
{
	/**
	 * Execute
	 *
	 * If ran successfully, should return anything worth logging. Only log something
	 * worth mentioning (don't log "task ran successfully"). Return NULL (actual NULL, not '' or 0) to not log (which will be most cases).
	 * If an error occurs which means the task could not finish running, throw an \IPS\Task\Exception - do not log an error as a normal log.
	 * Tasks should execute within the time of a normal HTTP request.
	 *
	 * @return	mixed	Message to log or NULL
	 * @throws	\IPS\Task\Exception
	 */
	public function execute()
	{
		if ( \IPS\Settings::i()->friends_notify_auto_del > 0 )
		{
			$select = \IPS\Db::i()->select( '*', 'friends_received_requests', array( 're_date<=?', time() - 86400 * \IPS\Settings::i()->friends_notify_auto_del ) );
			
			foreach( $select as $row )
			{
				\IPS\Db::i()->delete( 'friends_sent_requests', array( 'sent_from=? AND sent_to=?', $row['re_from'], $row['re_to'] ) );					
				\IPS\Db::i()->delete( 'friends_received_requests', array( 're_id=?', $row['re_id'] ) );	
			}
		}
		
		return NULL;
	}
	
	/**
	 * Cleanup
	 *
	 * If your task takes longer than 15 minutes to run, this method
	 * will be called before execute(). Use it to clean up anything which
	 * may not have been done
	 *
	 * @return	void
	 */
	public function cleanup()
	{
		
	}
}