<?php
/**
 * @brief		peopleYouMayKnow Widget
 * @author		<a href='https://www.invisioncommunity.com'>Invision Power Services, Inc.</a>
 * @copyright	(c) Invision Power Services, Inc.
 * @license		https://www.invisioncommunity.com/legal/standards/
 * @package		Invision Community
 * @subpackage	friends
 * @since		02 Jun 2020
 */

namespace IPS\friends\widgets;

/* To prevent PHP errors (extending class does not exist) revealing path */
if ( !\defined( '\IPS\SUITE_UNIQUE_KEY' ) )
{
	header( ( isset( $_SERVER['SERVER_PROTOCOL'] ) ? $_SERVER['SERVER_PROTOCOL'] : 'HTTP/1.0' ) . ' 403 Forbidden' );
	exit;
}

/**
 * peopleYouMayKnow Widget
 */
class _peopleYouMayKnow extends \IPS\Widget
{
	/**
	 * @brief	Widget Key
	 */
	public $key = 'peopleYouMayKnow';
	
	/**
	 * @brief	App
	 */
	public $app = 'friends';
		
	/**
	 * @brief	Plugin
	 */
	public $plugin = '';
	
	/**
	 * Initialise this widget
	 *
	 * @return void
	 */ 
	public function init()
	{
		// Use this to perform any set up and to assign a template that is not in the following format:
		$this->template( array( \IPS\Theme::i()->getTemplate( 'widgets', $this->app, 'front' ), $this->key ) );
		// If you are creating a plugin, uncomment this line:
		// $this->template( array( \IPS\Theme::i()->getTemplate( 'plugins', 'core', 'global' ), $this->key ) );
		// And then create your template at located at plugins/<your plugin>/dev/html/peopleYouMayKnow.phtml
		
		
		parent::init();
	}
	
	/**
	 * Specify widget configuration
	 *
	 * @param	null|\IPS\Helpers\Form	$form	Form object
	 * @return	null|\IPS\Helpers\Form
	 */
	public function configuration( &$form=null )
	{
 		$form = parent::configuration( $form );

 		$form->add( new \IPS\Helpers\Form\Text( 'friends_block_title', $this->configuration['friends_block_title'], FALSE ) );
 		$form->add( new \IPS\Helpers\Form\Number( 'friends_block_items', $this->configuration['friends_block_items'], TRUE, array( 'max' => 50 ) ) );
 		$form->add( new \IPS\Helpers\Form\Select( 'friends_block_style', $this->configuration['friends_block_style'], TRUE, array( 
			'options' => array( 'grid' => 'friends_block_style_grid', 'slider' => 'friends_block_style_slider' )
		) ) );
		
		$form->add( new \IPS\Helpers\Form\Select( 'friends_block_groups', isset( $this->configuration['friends_block_groups'] ) ? $this->configuration['friends_block_groups'] : '*', FALSE, array( 'options' => \IPS\Member\Group::groups(), 'parse' => 'normal', 'multiple' => TRUE ) ) );

	
 		return $form;
 	} 
 	
 	 /**
 	 * Ran before saving widget configuration
 	 *
 	 * @param	array	$values	Values from form
 	 * @return	array
 	 */
 	public function preConfig( $values )
 	{
 		return $values;
 	}

	/**
	 * Render a widget
	 *
	 * @return	string
	 */
	public function render()
	{
		# Check perm
		if ( \IPS\Member::loggedIn()->member_id == 0 || ! \IPS\Member::loggedIn()->canAccessModule( \IPS\Application\Module::get( 'friends', 'friends', 'front' ) ) )
		{
			return "";
		}
		
		$limit 	= $this->configuration['friends_block_items'] > 0 ? $this->configuration['friends_block_items'] : 10;
		$myFriends 	= array();
		$data 	= array();
		
		# Get my friends
		$mytotal = (int) \IPS\Db::i()->select( 'COUNT(*)', 'friends_friends', array( 'fi_owner=?', \IPS\Member::loggedIn()->member_id ) )->first();
		
		if ( $mytotal > 0 && $mytotal < \IPS\Settings::i()->friends_max )
		{
			# Check requests
			$req1 = iterator_to_array( \IPS\Db::i()->select( 're_from', 'friends_received_requests', array( 're_to=?', \IPS\Member::loggedIn()->member_id ) ) );
			$req2 = iterator_to_array( \IPS\Db::i()->select( 'sent_to', 'friends_sent_requests', array( 'sent_from=?', \IPS\Member::loggedIn()->member_id ) ) );
			$req = array_merge( $req1, $req2 );
			
			# Query
			$where[] = array( \IPS\Db::i()->in( 'fi_friend', $req, TRUE ) );
			
			$where[] = array( 'fi_friend!=?', \IPS\Member::loggedIn()->member_id  );
			$where[] = array( 'fi_owner!=?', \IPS\Member::loggedIn()->member_id  );			
			
			$where[] = array( 'fi_owner IN(?)', \IPS\Db::i()->select( 'fi_friend', 'friends_friends', array( 'fi_owner=?', \IPS\Member::loggedIn()->member_id ) ) );
			$where[] = array( 'fi_friend NOT IN(?)', \IPS\Db::i()->select( 'fi_friend', 'friends_friends', array( 'fi_owner=?', \IPS\Member::loggedIn()->member_id ) ) );

			# Ignored
			$ignored = iterator_to_array( \IPS\Db::i()->select( 'ignore_ignore_id, ignore_id', 'core_ignored_users', array( 'ignore_owner_id=?', \IPS\Member::loggedIn()->member_id ) )->setValueField( 'ignore_ignore_id' ) );

			if ( $ignored && \count( $ignored ) > 0 )
			{	
				$where[] = array( \IPS\Db::i()->in( 'fi_friend', implode( ",", $ignored ), true ) );
				$where[] = array( \IPS\Db::i()->in( 'fi_owner', implode( ",", $ignored ), true ) );
			}			
			
			$data = \IPS\Db::i()->select( 'DISTINCT fi_friend', 'friends_friends', $where, 'RAND()', $limit );

			if ( $data && \count( $data ) > 0 )
			{
				foreach( $data as $row )
				{
					$people[] = $row;
				}
						
				$memData = $this->getMemData( $people );
			}
		}

		return $this->output( $memData, $this->configuration );
	}
	
	protected function getMemData( $mems )
	{	
		$conf = $this->configuration;
		
		$where = array();

		$where[] = array( \IPS\Db::i()->in( 'member_id', array_unique( $mems ) ) );
		
		if ( isset( $conf['friends_block_groups'] ) && \count( $conf['friends_block_groups'] ) )
		{
			$where[] = array( \IPS\Db::i()->in( 'member_group_id', array_values( $conf['friends_block_groups'] ), TRUE ) );
		}

		foreach( \IPS\Db::i()->select( '*', 'core_members', $where, 'RAND()' ) as $mem )
		{
			$memData[] = \IPS\Member::constructFromData( $mem );
		}
		
		return $memData;
	}
	
	
	
	
	
	
	
	
	
}